@@ -2,7 +2,7 @@
 
 """
 import xml.etree.ElementTree as ET
-from .helpers import enum2str
+from ..helpers import enum2str
 from .enumerations import ObjectType, Orientation, Dynamic 
 
 class _SignalObjectBase():
@@ -140,9 +140,9 @@ class _SignalObjectBase():
             retdict['subtype'] = str(self.subtype)
             retdict['dynamic'] = enum2str(self.dynamic)
             retdict['zOffset'] = str(self.zOffset)
-            if self.pitch:
+            if self.pitch is not None:
                 retdict['pitch'] = str(self.pitch)
-            if self.roll:
+            if self.roll is not None:
                 retdict['roll'] = str(self.roll)
             if self.width is not None:
                 retdict['width'] = str(self.width)
@@ -184,7 +184,7 @@ class Signal(_SignalObjectBase):
             
             dynamic (Dynamic): specifies if Signal is static or dynamic (init in base class)
             
-            value (double): value for further specification of the signal
+            value (float): value for further specification of the signal
             
             unit (str): unit, needs to be provided when value is given
             
@@ -236,7 +236,7 @@ class Signal(_SignalObjectBase):
                 Default: None 
             dynamic (Dynamic): specifies if Signal is static or dynamic (init in base class)
                 Default: Dynamic.no 
-            value (double): value for further specification of the signal
+            value (float): value for further specification of the signal
                 Default: None
             unit (str): unit, needs to be provided when value is given
                 Default: None                                  
@@ -285,7 +285,7 @@ class Signal(_SignalObjectBase):
         retdict["country"] = str(self.country).upper()
         retdict["type"] = str(self.type)
         retdict["subtype"] = str(self.subtype)
-        if self.hOffset:
+        if self.hOffset is not None:
             retdict["hOffset"] = str(self.hOffset)    
         #TODO check if value is supplied --> unit is mandatory in that case   
         if self.value is not None:
