@@ -2,13 +2,14 @@
 
 """
 
+from .position import _PositionFactory
 from .exceptions import OpenSCENARIOVersionError, NotEnoughInputArguments
 
 import xml.etree.ElementTree as ET
 import xml.dom.minidom as mini
 
 
-from .helpers import printToFile
+from ..helpers import printToFile
 from .utils import FileHeader, ParameterDeclarations, Catalog, TrafficSignalController
 from .enumerations import VersionBase, XMLNS, XSI
 from .entities import Entities
@@ -35,6 +36,9 @@ class Scenario():
                 Default: 1
         Methods
         -------
+            parse(element)
+                parses a ElementTree created by the class and returns an instance of the class
+
             get_element()
                 Returns the full ElementTree of the class
 
@@ -82,7 +86,7 @@ class Scenario():
         self.roadnetwork = roadnetwork
         self.catalog = catalog 
         self.parameters = parameters
-        self.header = FileHeader(name,author,revMinor=osc_minor_version)
+        self.header = FileHeader(author,name,revMinor=osc_minor_version)
 
     def __eq__(self,other):
         if isinstance(other,Scenario):
@@ -95,6 +99,30 @@ class Scenario():
                 return True
         return False
 
+    @staticmethod 
+    def parse(element):
+        """ Parses the xml element of Scenario
+
+            Parameters
+            ----------
+                element (xml.etree.ElementTree.Element): A Scenario element (same as generated by the class itself)
+
+            Returns
+            -------
+                scenario (Scenario): a Scenario object
+
+        """
+        header = FileHeader.parse(element.find('FileHeader'))
+        parameters = ParameterDeclarations.parse(element.find('ParameterDeclarations'))
+        catalog = Catalog.parse(element.find('CatalogLocations'))
+        storyboard = StoryBoard.parse(element.find('Storyboard'))
+        entities = Entities.parse(element.find('Entities'))
+        roadnetwork = RoadNetwork.parse(element.find('RoadNetwork'))
+
+        return Scenario(header.description, header.author, parameters,entities,storyboard,roadnetwork,catalog,header._revMinor)
+        
+
+
     def get_element(self):
         """ returns the elementTree of the Scenario
 
@@ -109,8 +137,8 @@ class Scenario():
 
         return element
 
-    def write_xml(self,filename,prettyprint = True):
-        """ writeXml writes the open scenario xml file
+    def write_xml(self, filename, prettyprint = True, encoding = 'utf-8'):
+        """ write_xml writes the OpenSCENARIO xml file
 
         Parameters
         ----------
@@ -118,6 +146,9 @@ class Scenario():
 
             prettyprint (bool): pretty print or ugly print?
                 Default: True
+                
+            encoding (str): specifies the output encoding
+                Default: 'utf-8'
 
         """
         printToFile(self.get_element(),filename,prettyprint)
@@ -146,18 +177,21 @@ class RoadNetwork(VersionBase):
             
         Methods
         -------
+            parse(element)
+                parses a ElementTree created by the class and returns an instance of the class
+
             get_element()
                 Returns the full ElementTree of the class
 
 
 
     """
-    def __init__(self,roadfile,scenegraph=None):
+    def __init__(self,roadfile=None,scenegraph=None):
         """ Initalizes the RoadNetwork
 
         Parameters
         ----------
-            roadfile (str): path to the opendrive file
+            roadfile (str): path to the opendrive file (optional)
 
             scenegraph (str): path to the opensceengraph file (optional)
 
@@ -171,10 +205,48 @@ class RoadNetwork(VersionBase):
         if isinstance(other,RoadNetwork):
             if self.road_file == other.road_file and \
             self.scene == other.scene and \
-            set(self.traffic_signals) == set(other.traffic_signals):
+            self.traffic_signals == other.traffic_signals and \
+            self.used_area_positions == other.used_area_positions:
                 return True
         return False
 
+    @staticmethod 
+    def parse(element):
+        """ Parses the xml element of RoadNetwork
+
+            Parameters
+            ----------
+                element (xml.etree.ElementTree.Element): A RoadNetwork element (same as generated by the class itself)
+
+            Returns
+            -------
+                roadnetwork (RoadNetwork): a RoadNetwork object
+
+        """
+        logicFile = None
+        if element.find('LogicFile') is not None:
+            logicFile = element.find('LogicFile').attrib['filepath']
+        
+        sceneGraphFile = None
+        if element.find('SceneGraphFile') is not None:
+            sceneGraphFile = element.find('SceneGraphFile').attrib['filepath']
+
+        roadnetwork = RoadNetwork(roadfile=logicFile, scenegraph= sceneGraphFile)
+
+        position_elements = element.findall('UsedArea/Position')
+        if position_elements is not None:
+            for position in position_elements:
+                roadnetwork.add_used_area_position(_PositionFactory.parse_position(position))
+
+        tsc_elements = element.findall('TrafficSignals/TrafficSignalController')
+        if tsc_elements is not None:
+            for tsc_element in tsc_elements:
+                controller = TrafficSignalController.parse(tsc_element)
+                roadnetwork.add_traffic_signal_controller(controller)
+        
+        return roadnetwork
+
+
     def add_traffic_signal_controller(self,traffic_signal_controller):
         """ adds a TrafficSignalController to the RoadNetwork
 
@@ -203,7 +275,8 @@ class RoadNetwork(VersionBase):
 
         """
         roadnetwork = ET.Element('RoadNetwork')
-        ET.SubElement(roadnetwork,'LogicFile',{'filepath': self.road_file})
+        if self.road_file:
+            ET.SubElement(roadnetwork,'LogicFile',{'filepath': self.road_file})
         if self.scene:
             ET.SubElement(roadnetwork,'SceneGraphFile',{'filepath':self.scene})
         if self.traffic_signals:
