@@ -475,3 +475,59 @@ def test_junction_group():
     assert jg == jg2
     assert jg != jg3
     
+def test_lanelinking_roads_pre_suc():
+    road1 = pyodrx.create_road(pyodrx.Line(10),0,1,1)
+    road2 = pyodrx.create_road(pyodrx.Line(10),1,1,1)
+    road1.add_successor(pyodrx.ElementType.road,1,pyodrx.ContactPoint.start)
+    road2.add_predecessor(pyodrx.ElementType.road,0,pyodrx.ContactPoint.end)
+    odr = pyodrx.OpenDrive('test')
+    odr.add_road(road1)
+    odr.add_road(road2)
+    odr.adjust_roads_and_lanes()
+    assert road1.lanes.lanesections[0].leftlanes[0].links.links[0].link_type == 'successor'
+    assert road1.lanes.lanesections[0].rightlanes[0].links.links[0].link_type == 'successor'
+    assert road1.lanes.lanesections[0].leftlanes[0].links.links[0].element_id == '1'
+    assert road1.lanes.lanesections[0].rightlanes[0].links.links[0].element_id == '-1'
+
+    assert road2.lanes.lanesections[0].leftlanes[0].links.links[0].link_type == 'predecessor'
+    assert road2.lanes.lanesections[0].rightlanes[0].links.links[0].link_type == 'predecessor'
+    assert road2.lanes.lanesections[0].leftlanes[0].links.links[0].element_id == '1'
+    assert road2.lanes.lanesections[0].rightlanes[0].links.links[0].element_id == '-1'
+
+def test_lanelinking_roads_suc_suc():
+    road1 = pyodrx.create_road(pyodrx.Line(10),0,1,1)
+    road2 = pyodrx.create_road(pyodrx.Line(10),1,1,1)
+    road1.add_successor(pyodrx.ElementType.road,1,pyodrx.ContactPoint.end)
+    road2.add_successor(pyodrx.ElementType.road,0,pyodrx.ContactPoint.end)
+    odr = pyodrx.OpenDrive('test')
+    odr.add_road(road1)
+    odr.add_road(road2)
+    odr.adjust_roads_and_lanes()
+    assert road1.lanes.lanesections[0].leftlanes[0].links.links[0].link_type == 'successor'
+    assert road1.lanes.lanesections[0].rightlanes[0].links.links[0].link_type == 'successor'
+    assert road1.lanes.lanesections[0].leftlanes[0].links.links[0].element_id == '-1'
+    assert road1.lanes.lanesections[0].rightlanes[0].links.links[0].element_id == '1'
+
+    assert road2.lanes.lanesections[0].leftlanes[0].links.links[0].link_type == 'successor'
+    assert road2.lanes.lanesections[0].rightlanes[0].links.links[0].link_type == 'successor'
+    assert road2.lanes.lanesections[0].leftlanes[0].links.links[0].element_id == '-1'
+    assert road2.lanes.lanesections[0].rightlanes[0].links.links[0].element_id == '1'
+
+def test_lanelinking_roads_suc_suc():
+    road1 = pyodrx.create_road(pyodrx.Line(10),0,1,1)
+    road2 = pyodrx.create_road(pyodrx.Line(10),1,1,1)
+    road1.add_predecessor(pyodrx.ElementType.road,1,pyodrx.ContactPoint.start)
+    road2.add_predecessor(pyodrx.ElementType.road,0,pyodrx.ContactPoint.start)
+    odr = pyodrx.OpenDrive('test')
+    odr.add_road(road1)
+    odr.add_road(road2)
+    odr.adjust_roads_and_lanes()
+    assert road1.lanes.lanesections[0].leftlanes[0].links.links[0].link_type == 'predecessor'
+    assert road1.lanes.lanesections[0].rightlanes[0].links.links[0].link_type == 'predecessor'
+    assert road1.lanes.lanesections[0].leftlanes[0].links.links[0].element_id == '-1'
+    assert road1.lanes.lanesections[0].rightlanes[0].links.links[0].element_id == '1'
+
+    assert road2.lanes.lanesections[0].leftlanes[0].links.links[0].link_type == 'predecessor'
+    assert road2.lanes.lanesections[0].rightlanes[0].links.links[0].link_type == 'predecessor'
+    assert road2.lanes.lanesections[0].leftlanes[0].links.links[0].element_id == '-1'
+    assert road2.lanes.lanesections[0].rightlanes[0].links.links[0].element_id == '1'
\ No newline at end of file
